from django.test import TestCase
from django.test.client import RequestFactory
from django.http import HttpResponse, QueryDict
from django.utils.decorators import method_decorator

from ..decorators import app_proxy
from ..helpers import get_proxy_signature


class AppProxyTestCase(TestCase):

    def setUp(self):
        self.request_factory = RequestFactory()

    @method_decorator(app_proxy)
    def proxy_view(self, request, *args, **kwargs):
        return HttpResponse('OK')

    def get_proxy_request(self, signature = None):
        # Add common parameters to the request.
        data = {
            'extra': ['1', '2'],
            'shop': 'shop-name.myshopify.com',
            'path_prefix': '/apps/awesome_reviews',
            'timestamp': '1317327555'
        }

        if signature is not None:
            data['signature'] = signature

        request = self.request_factory.get('/proxy/', data = data)
        return request

    def test_signature_calculation(self):
        # Use the example provided in the Shopify documentation to verify our signature calculation works correctly.
        # See: http://docs.shopify.com/api/tutorials/application-proxies#security
        query_dict = QueryDict('extra=1&extra=2&shop=shop-name.myshopify.com&path_prefix=%2Fapps%2Fawesome_reviews&timestamp=1317327555')
        shared_secret = 'hush'
        expected_signature = 'a9718877bea71c2484f91608a7eaea1532bdf71f5c56825065fa4ccabe549ef3'
        calculated_signature = get_proxy_signature(query_dict, shared_secret)
        self.assertEqual(calculated_signature, expected_signature)

    def test_missing_signature_is_bad_request(self):
        request = self.get_proxy_request()
        response = self.proxy_view(request)
        self.assertEqual(response.status_code, 400)

    def test_invalid_signature_is_bad_request(self):
        request = self.get_proxy_request(signature = 'invalid')
        response = self.proxy_view(request)
        self.assertEqual(response.status_code, 400)

    def test_valid_signature_is_ok(self):
        request = self.get_proxy_request(signature = 'a9718877bea71c2484f91608a7eaea1532bdf71f5c56825065fa4ccabe549ef3')
        response = self.proxy_view(request)
        self.assertEqual(response.status_code, 200)


from django.dispatch import Signal


class WebhookSignal(Signal):
    """
    A class wrapping Signal with the common arguments for Webhooks.
    """

    def __init__(self):
        providing_args = ['domain', 'topic', 'data']
        super(WebhookSignal, self).__init__(providing_args = providing_args)


# Define a generic webhook_received signal that triggers for all webhooks.
webhook_received = WebhookSignal()


# Define topic-specific signals.
orders_create = WebhookSignal()
orders_delete = WebhookSignal()
orders_updated = WebhookSignal()
orders_paid = WebhookSignal()
orders_cancelled = WebhookSignal()
orders_fulfilled = WebhookSignal()
orders_partially_fulfilled = WebhookSignal()
order_transactions_create = WebhookSignal()
carts_create = WebhookSignal()
carts_update = WebhookSignal()
checkouts_create = WebhookSignal()
checkouts_update = WebhookSignal()
checkouts_delete = WebhookSignal()
refunds_create = WebhookSignal()
products_create = WebhookSignal()
products_update = WebhookSignal()
products_delete = WebhookSignal()
collections_create = WebhookSignal()
collections_update = WebhookSignal()
collections_delete = WebhookSignal()
customer_groups_create = WebhookSignal()
customer_groups_update = WebhookSignal()
customer_groups_delete = WebhookSignal()
customers_create = WebhookSignal()
customers_enable = WebhookSignal()
customers_disable = WebhookSignal()
customers_update = WebhookSignal()
customers_delete = WebhookSignal()
fulfillments_create = WebhookSignal()
fulfillments_update = WebhookSignal()
shop_update = WebhookSignal()
disputes_create = WebhookSignal()
disputes_update = WebhookSignal()
app_uninstalled = WebhookSignal()



from django.dispatch import receiver

from ..signals import webhook_received, orders_create
from . import WebhookTestCase


class WebhookViewTestCase(WebhookTestCase):

    def test_get_method_not_allowed(self):
        response = self.client.get(self.webhook_url)
        self.assertEqual(response.status_code, 405, 'GET request returns 405 (Method Not Allowed).')

    def test_empty_post_message_is_bad_request(self):
        response = self.post_shopify_webhook()
        self.assertEqual(response.status_code, 400, 'Empty POST request returns 400 (Bad Request).')

    def test_no_hmac_is_forbidden(self):
        response = self.post_shopify_webhook(topic = 'orders/create', data = {'id': 123}, send_hmac = False)
        self.assertEqual(response.status_code, 403, 'POST orders/create request with no HMAC returns 403 (Forbidden).')

    def test_invalid_hmac_is_forbidden(self):
        response = self.post_shopify_webhook(topic = 'orders/create', data = {'id': 123}, headers = {'HTTP_X_SHOPIFY_HMAC_SHA256': 'invalid'}, send_hmac = False)
        self.assertEqual(response.status_code, 403, 'POST orders/create request with invalid HMAC returns 403 (Forbidden).')

    def test_unknown_topic_is_bad_request(self):
        response = self.post_shopify_webhook(topic = 'tests/invalid', data = {'id': 123})
        self.assertEqual(response.status_code, 400, 'POST tests/invalid request with valid HMAC returns 400 (Bad Request).')

    def test_missing_domain_is_bad_request(self):
        response = self.post_shopify_webhook(topic = 'orders/create', domain = '', data = {'id': 123})
        self.assertEqual(response.status_code, 400, 'POST orders/create request with missing domain returns 400 (Bad Request).')

    def test_valid_hmac_is_ok(self):
        response = self.post_shopify_webhook(topic = 'orders/create', data = {'id': 123})
        self.assertEqual(response.status_code, 200, 'POST orders/create request with valid HMAC returns 200 (OK).')

    def test_webook_received_signal_triggered(self):
        data = {'id': 123456}

        # Create a test signal receiver for the generic webhook received signal.
        @receiver(webhook_received)
        def test_webhook_received_receiver(sender, data, **kwargs):
            test_webhook_received_receiver.data = data
        test_webhook_received_receiver.data = None

        response = self.post_shopify_webhook(topic = 'fulfillments/update', data = data)
        self.assertEqual(data, test_webhook_received_receiver.data, 'POST fulfillments/update correctly triggered webhook_received signal.')

    def test_order_created_signal_triggered(self):
        data = {'id': 123456}

        # Create a test signal receiver for the order/created topic.
        @receiver(orders_create)
        def test_order_create_receiver(sender, data, **kwargs):
            test_order_create_receiver.data = data
        test_order_create_receiver.data = None

        response = self.post_shopify_webhook(topic = 'orders/create', data = data)
        self.assertEqual(data, test_order_create_receiver.data, 'POST orders/create correctly triggered order_created signal.')



import os, sys
import json

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings
from django.utils import six

from ..helpers import get_hmac


class WebhookTestCase(TestCase):
    """
    A base class for running tests on Shopify webhooks. Can be used by `shopify_webhook` tests here, or by other
    packages that utilise webhook behaviour.
    """

    def setUp(self):
        """
        Set up the test case, primarily by getting a reference to the webhook endpoint to be used for testing.
        """
        super(WebhookTestCase, self).setUp()
        self.webhook_url = reverse('webhook')

    def post_shopify_webhook(self, topic = None, domain = None, data = None, headers = None, send_hmac = True):
        """
        Simulate a webhook being sent to the application's webhook endpoint with the provided parameters.
        """
        # Set defaults.
        domain = 'test.myshopify.com' if domain is None else domain
        data = {} if data is None else data
        headers = {} if headers is None else headers

        # Dump data as a JSON string.
        data = json.dumps(data)

        # Add required headers.
        headers['HTTP_X_SHOPIFY_TEST'] = 'true'
        headers['HTTP_X_SHOPIFY_SHOP_DOMAIN'] = domain

        # Add optional headers.
        if topic:
            headers['HTTP_X_SHOPIFY_TOPIC'] = topic
        if send_hmac:
            headers['HTTP_X_SHOPIFY_HMAC_SHA256'] = six.text_type(get_hmac(six.b(data), settings.SHOPIFY_APP_API_SECRET))

        return self.client.post(self.webhook_url, data = data, content_type = 'application/json', **headers)

    def read_fixture(self, name):
        """
        Read a .json fixture with the specified name, parse it as JSON and return.
        Currently makes the assumption that a directory named 'fixtures' containing .json files exists and is located
        in the same directory as the file running the tests.
        """
        fixture_path = "{0}/fixtures/{1}.json".format(os.path.dirname(sys.modules[self.__module__].__file__), name, format)
        with open(fixture_path, 'rb') as f:
            return json.loads(f.read())
